# BambangShop Publisher App
Tutorial and Example for Advanced Programming 2024 - Faculty of Computer Science, Universitas Indonesia

---

## About this Project
In this repository, we have provided you a REST (REpresentational State Transfer) API project using Rocket web framework.

This project consists of four modules:
1.  `controller`: this module contains handler functions used to receive request and send responses.
    In Model-View-Controller (MVC) pattern, this is the Controller part.
2.  `model`: this module contains structs that serve as data containers.
    In MVC pattern, this is the Model part.
3.  `service`: this module contains structs with business logic methods.
    In MVC pattern, this is also the Model part.
4.  `repository`: this module contains structs that serve as databases and methods to access the databases.
    You can use methods of the struct to get list of objects, or operating an object (create, read, update, delete).

This repository provides a basic functionality that makes BambangShop work: ability to create, read, and delete `Product`s.
This repository already contains a functioning `Product` model, repository, service, and controllers that you can try right away.

As this is an Observer Design Pattern tutorial repository, you need to implement another feature: `Notification`.
This feature will notify creation, promotion, and deletion of a product, to external subscribers that are interested of a certain product type.
The subscribers are another Rocket instances, so the notification will be sent using HTTP POST request to each subscriber's `receive notification` address.

## API Documentations

You can download the Postman Collection JSON here: https://ristek.link/AdvProgWeek7Postman

After you download the Postman Collection, you can try the endpoints inside "BambangShop Publisher" folder.
This Postman collection also contains endpoints that you need to implement later on (the `Notification` feature).

Postman is an installable client that you can use to test web endpoints using HTTP request.
You can also make automated functional testing scripts for REST API projects using this client.
You can install Postman via this website: https://www.postman.com/downloads/

## How to Run in Development Environment
1.  Set up environment variables first by creating `.env` file.
    Here is the example of `.env` file:
    ```bash
    APP_INSTANCE_ROOT_URL="http://localhost:8000"
    ```
    Here are the details of each environment variable:
    | variable              | type   | description                                                |
    |-----------------------|--------|------------------------------------------------------------|
    | APP_INSTANCE_ROOT_URL | string | URL address where this publisher instance can be accessed. |
2.  Use `cargo run` to run this app.
    (You might want to use `cargo check` if you only need to verify your work without running the app.)

## Mandatory Checklists (Publisher)
-   [ ] Clone https://gitlab.com/ichlaffterlalu/bambangshop to a new repository.
-   **STAGE 1: Implement models and repositories**
    -   [ ] Commit: `Create Subscriber model struct.`
    -   [ ] Commit: `Create Notification model struct.`
    -   [ ] Commit: `Create Subscriber database and Subscriber repository struct skeleton.`
    -   [ ] Commit: `Implement add function in Subscriber repository.`
    -   [ ] Commit: `Implement list_all function in Subscriber repository.`
    -   [ ] Commit: `Implement delete function in Subscriber repository.`
    -   [ ] Write answers of your learning module's "Reflection Publisher-1" questions in this README.
-   **STAGE 2: Implement services and controllers**
    -   [ ] Commit: `Create Notification service struct skeleton.`
    -   [ ] Commit: `Implement subscribe function in Notification service.`
    -   [ ] Commit: `Implement subscribe function in Notification controller.`
    -   [ ] Commit: `Implement unsubscribe function in Notification service.`
    -   [ ] Commit: `Implement unsubscribe function in Notification controller.`
    -   [ ] Write answers of your learning module's "Reflection Publisher-2" questions in this README.
-   **STAGE 3: Implement notification mechanism**
    -   [ ] Commit: `Implement update method in Subscriber model to send notification HTTP requests.`
    -   [ ] Commit: `Implement notify function in Notification service to notify each Subscriber.`
    -   [ ] Commit: `Implement publish function in Program service and Program controller.`
    -   [ ] Commit: `Edit Product service methods to call notify after create/delete.`
    -   [ ] Write answers of your learning module's "Reflection Publisher-3" questions in this README.

## Your Reflections
This is the place for you to write reflections:

### Mandatory (Publisher) Reflections

#### Reflection Publisher-1
## 1. Do we still need an interface (or trait in Rust) for the Subscriber in the BambangShop case?

In the Observer design pattern, the `Subscriber` (or `Observer`) is typically an interface (trait in Rust) that allows different concrete implementations of subscribers to be notified when a state change occurs in the `Subject`. This abstraction enables flexibility, as multiple types of subscribers can exist with different behaviors. However, in the BambangShop case, if all subscribers share the same structure and behavior meaning they only store information and do not require different implementations then using a single `Subscriber` struct is sufficient, and a trait is unnecessary. If, in the future, different types of subscribers with distinct behaviors (e.g., logging subscribers, email subscribers, etc.) are needed, then defining a `Subscriber` trait would make sense. Otherwise, keeping it as a single struct simplifies the design without unnecessary abstraction.

## 2. Is using `Vec` sufficient, or is `DashMap` necessary for unique `id` in `Program` and `url` in `Subscriber`?

Using a `Vec` (list) would not be an efficient solution for ensuring uniqueness of `id` in `Program` and `url` in `Subscriber`, because checking for uniqueness in a `Vec` requires iterating through all elements, making insertions and lookups have a time complexity of `O(n)`. In contrast, `DashMap` (or a `HashMap` in a single-threaded context) provides constant-time `O(1)` average complexity for insertions and lookups, ensuring uniqueness efficiently. Since the requirement is to enforce unique `id` and `url`, a map-based structure like `DashMap` is necessary. Additionally, `DashMap` allows for concurrent read and write operations, making it more suitable in a multi-threaded environment where multiple threads may access or modify the subscriber list simultaneously.

## 3. Do we still need `DashMap`, or can we use the Singleton pattern instead?

The Singleton pattern ensures that only one instance of a structure exists throughout the program, which is useful for maintaining a global state, such as the `SUBSCRIBERS` list. However, in Rust, simply using the Singleton pattern with a `Mutex<HashMap>` or `RwLock<HashMap>` would introduce locking overhead, which can degrade performance when multiple threads try to read or write simultaneously. `DashMap` is designed to provide fine-grained locking, allowing multiple threads to access different parts of the map concurrently without blocking each other. This makes `DashMap` a more efficient and scalable solution for a globally shared, thread-safe `HashMap`. While the Singleton pattern could be used, it would require additional synchronization mechanisms, making `DashMap` the better choice for managing concurrent access to subscribers in a high-performance application.

#### Reflection Publisher-2
### 1. Why Separate "Service" and "Repository" from a Model in MVC?

Separating "Service" and "Repository" from the Model in MVC follows the **Separation of Concerns** principle, which improves maintainability and scalability. The **Repository** is responsible for interacting with the database, handling queries, and ensuring efficient data retrieval, while the **Service** layer processes business logic and enforces rules. This separation prevents the Model from becoming bloated with both business logic and persistence logic. Additionally, it enhances testability, as we can test business logic independently from the database. It also increases flexibility, making it easier to switch database implementations without modifying business logic. Encapsulation ensures that database operations do not leak into other parts of the application, leading to better security and stability. Without these layers, modifying one aspect of the system such as changing the database structure would require altering the entire Model, making the code harder to maintain and extend.

### 2. What Happens if We Only Use the Model?

If we only use the Model without separating the Service and Repository layers, the system quickly becomes tightly coupled and difficult to manage. The Model would need to handle database queries, business logic, and possibly even API interactions, leading to a complex and rigid structure. As the application grows, interactions between different models such as `Program`, `Subscriber`, and `Notification` would create tangled dependencies. For example, if `Notification` needs data from `Subscriber` but `Subscriber` also updates `Notification`, maintaining such relationships directly within the Model would increase code complexity and risk unintended side effects. Debugging and modifying such a system would be challenging since any change in one model could have unpredictable consequences across others. Additionally, testing would become cumbersome because unit tests would need to interact with the database, slowing down development and making the codebase more fragile.

### 3. Exploring Postman for API Testing

Postman is an invaluable tool for API development and testing, allowing developers to efficiently test their endpoints without needing to build a full frontend application. It enables sending HTTP requests such as GET, POST, PUT, and DELETE to interact with APIs, making debugging much faster. One of the most useful features is the ability to **automate tests** by writing pre-request and post-request scripts, ensuring API responses behave as expected. Postman also provides **environment management**, which allows switching between different API environments (development, staging, production) using variables. Additionally, the **Collection Runner** feature enables executing multiple API calls in sequence, which is beneficial for testing workflows such as authentication or transactions. In my current work and group project, Postman helps in validating API endpoints, testing authentication mechanisms, and documenting API specifications for better collaboration. Features like **mock servers, automated testing, and scheduled API monitoring** make it an essential tool for ensuring API reliability in future software engineering projects.


#### Reflection Publisher-3
### 1. Observer Pattern Variation Used in This Tutorial  
In this tutorial, we use the **Push model** of the Observer Pattern. The `notify` function constructs a `Notification` payload and directly sends it to all subscribers using `subscriber_clone.update(payload_clone)`. This means the publisher (the system handling notifications) proactively pushes data to subscribers without them requesting it.

### 2. Advantages and Disadvantages of Using the Pull Model  
If we used the **Pull model** instead, subscribers would request updates from the publisher rather than receiving notifications automatically. One advantage of this approach is that it reduces unnecessary notifications subscribers fetch updates only when needed. This can be beneficial in systems with many inactive subscribers. However, a major downside is increased complexity and potential delays. Subscribers must continuously check for updates, leading to inefficient polling, wasted resources, and a slower response time compared to the Push model.

### 3. Impact of Not Using Multi-threading in the Notification Process  
If we remove multi-threading from the notification process, the system will notify each subscriber sequentially, meaning one update must complete before moving to the next. This could lead to significant delays, especially if a subscriber's update request is slow or times out. In contrast, with multi-threading, each subscriber is notified independently and concurrently, improving efficiency and ensuring that slow subscribers do not block the notification process for others.
